@@ -3,7 +3,7 @@
  *
  * Copyright (C) 2005-2012  NTT DATA CORPORATION
  *
- * Version: 0.1.11   2015/01/12
+ * Version: 0.1.12   2015/04/08
  */
 
 #ifndef _SECURITY_CAITSITH_INTERNAL_H
@@ -871,7 +871,7 @@ bool cs_manager(void);
 bool cs_transit_domain(const char *domainname);
 char *cs_encode(const char *str);
 char *cs_encode2(const char *str, int str_len);
-char *cs_realpath(struct path *path);
+char *cs_realpath(const struct path *path);
 char *cs_get_exe(void);
 int cs_audit_log(struct cs_request_info *r);
 int cs_check_acl(struct cs_request_info *r, const bool clear);
@@ -882,30 +882,31 @@ void cs_notify_gc(struct cs_io_buffer *head, const bool is_register);
 void cs_populate_patharg(struct cs_request_info *r, const bool first);
 void cs_transition_failed(const char *domainname);
 void cs_warn_oom(const char *function);
-int cs_chroot_permission(struct path *path);
-int cs_chmod_permission(struct path *path, mode_t mode);
+int cs_chroot_permission(const struct path *path);
+int cs_chmod_permission(const struct path *path, mode_t mode);
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(3, 5, 0)
-int cs_chown_permission(struct path *path, kuid_t user, kgid_t group);
+int cs_chown_permission(const struct path *path, kuid_t user, kgid_t group);
 #else
-int cs_chown_permission(struct path *path, uid_t user, gid_t group);
+int cs_chown_permission(const struct path *path, uid_t user, gid_t group);
 #endif
 int cs_fcntl_permission(struct file *file, unsigned int cmd,
 			unsigned long arg);
 int cs_ioctl_permission(struct file *filp, unsigned int cmd,
 			unsigned long arg);
-int cs_link_permission(struct path *old, struct path *new);
-int cs_mkdir_permission(struct path *path, unsigned int mode);
-int cs_mknod_permission(struct path *path, const unsigned int mode,
+int cs_link_permission(const struct path *old, const struct path *new);
+int cs_mkdir_permission(const struct path *path, unsigned int mode);
+int cs_mknod_permission(const struct path *path, const unsigned int mode,
 			unsigned int dev);
-int cs_mount_permission(const char *dev_name, struct path *path,
+int cs_mount_permission(const char *dev_name, const struct path *path,
 			const char *type, unsigned long flags,
 			void *data_page);
-int cs_pivot_root_permission(struct path *old_path, struct path *new_path);
-int cs_rename_permission(struct path *old, struct path *new);
-int cs_symlink_permission(struct path *path, const char *from);
-int cs_truncate_permission(struct path *path);
-int cs_umount_permission(struct path *path, int flags);
-int cs_unlink_permission(struct path *path);
+int cs_pivot_root_permission(const struct path *old_path,
+			     const struct path *new_path);
+int cs_rename_permission(const struct path *old, const struct path *new);
+int cs_symlink_permission(const struct path *path, const char *from);
+int cs_truncate_permission(const struct path *path);
+int cs_umount_permission(const struct path *path, int flags);
+int cs_unlink_permission(const struct path *path);
 int cs_socket_create_permission(int family, int type, int protocol);
 int cs_socket_bind_permission(struct socket *sock, struct sockaddr *addr,
 			      int addr_len);
@@ -916,10 +917,10 @@ int cs_socket_post_accept_permission(struct socket *sock,
 				     struct socket *newsock);
 int cs_socket_sendmsg_permission(struct socket *sock,
 				 struct msghdr *msg, int size);
-int cs_rmdir_permission(struct path *path);
-int cs_getattr_permission(struct path *path);
+int cs_rmdir_permission(const struct path *path);
+int cs_getattr_permission(const struct path *path);
 bool cs_capable(const u8 operation);
-int cs_open_permission(struct path *path, const int flag);
+int cs_open_permission(const struct path *path, const int flag);
 
 /* Variable definition for internal use. */
 
