@@ -3,7 +3,7 @@
  *
  * Copyright (C) 2005-2012  NTT DATA CORPORATION
  *
- * Version: 0.1.11   2015/01/12
+ * Version: 0.1.12   2015/04/08
  */
 
 #ifndef _SECURITY_CAITSITH_INTERNAL_H
@@ -862,7 +862,7 @@ bool cs_manager(void);
 bool cs_transit_domain(const char *domainname);
 char *cs_encode(const char *str);
 char *cs_encode2(const char *str, int str_len);
-char *cs_realpath(struct path *path);
+char *cs_realpath(const struct path *path);
 char *cs_get_exe(void);
 int cs_audit_log(struct cs_request_info *r);
 int cs_check_acl(struct cs_request_info *r, const bool clear);
