@@ -3,7 +3,7 @@
  *
  * Copyright (C) 2005-2012  NTT DATA CORPORATION
  *
- * Version: 0.1.11   2015/01/12
+ * Version: 0.1.12   2015/04/08
  */
 
 #ifndef _LINUX_CAITSITH_H
@@ -64,10 +64,10 @@ struct caitsith_exports {
 /* For doing access control. */
 struct caitsith_operations {
 	void (*check_profile) (void);
-	int (*chroot_permission) (struct path *path);
-	int (*pivot_root_permission) (struct path *old_path,
-				      struct path *new_path);
-	int (*mount_permission) (const char *dev_name, struct path *path,
+	int (*chroot_permission) (const struct path *path);
+	int (*pivot_root_permission) (const struct path *old_path,
+				      const struct path *new_path);
+	int (*mount_permission) (const char *dev_name, const struct path *path,
 				 const char *type, unsigned long flags,
 				 void *data_page);
 	int (*umount_permission) (struct vfsmount *mnt, int flags);
@@ -152,26 +152,27 @@ struct caitsith_operations {
 
 extern struct caitsith_operations caitsith_ops;
 
-static inline int ccs_chroot_permission(struct path *path)
+static inline int ccs_chroot_permission(const struct path *path)
 {
-	int (*func) (struct path *) = caitsith_ops.chroot_permission;
+	int (*func) (const struct path *) = caitsith_ops.chroot_permission;
 	return func ? func(path) : 0;
 }
 
-static inline int ccs_pivot_root_permission(struct path *old_path,
-					    struct path *new_path)
+static inline int ccs_pivot_root_permission(const struct path *old_path,
+					    const struct path *new_path)
 {
-	int (*func) (struct path *, struct path *)
+	int (*func) (const struct path *, const struct path *)
 		= caitsith_ops.pivot_root_permission;
 	return func ? func(old_path, new_path) : 0;
 }
 
-static inline int ccs_mount_permission(const char *dev_name, struct path *path,
+static inline int ccs_mount_permission(const char *dev_name,
+				       const struct path *path,
 				       const char *type, unsigned long flags,
 				       void *data_page)
 {
-	int (*func) (const char *, struct path *, const char *, unsigned long,
-		     void *) = caitsith_ops.mount_permission;
+	int (*func) (const char *, const struct path *, const char *,
+		     unsigned long, void *) = caitsith_ops.mount_permission;
 	return func ? func(dev_name, path, type, flags, data_page) : 0;
 }
 
@@ -380,18 +381,19 @@ static inline int ccs_search_binary_handler(struct linux_binprm *bprm,
 
 #else
 
-static inline int ccs_chroot_permission(struct path *path)
+static inline int ccs_chroot_permission(const struct path *path)
 {
 	return 0;
 }
 
-static inline int ccs_pivot_root_permission(struct path *old_path,
-					    struct path *new_path)
+static inline int ccs_pivot_root_permission(const struct path *old_path,
+					    const struct path *new_path)
 {
 	return 0;
 }
 
-static inline int ccs_mount_permission(const char *dev_name, struct path *path,
+static inline int ccs_mount_permission(const char *dev_name,
+				       const struct path *path,
 				       const char *type, unsigned long flags,
 				       void *data_page)
 {
