@@ -3,7 +3,7 @@
  *
  * Copyright (C) 2010-2013  Tetsuo Handa <penguin-kernel@I-love.SAKURA.ne.jp>
  *
- * Version: 0.1.11   2015/01/12
+ * Version: 0.1.12   2015/04/08
  */
 
 #include "caitsith.h"
@@ -50,7 +50,9 @@ static struct cs_security cs_default_security = {
 struct caitsith_exports caitsith_exports;
 
 /* Original hooks. */
-static struct security_operations original_security_ops;
+static union security_list_options original_cred_prepare;
+static union security_list_options original_cred_free;
+static union security_list_options original_cred_alloc_blank;
 
 #if !defined(CONFIG_CAITSITH_DEBUG)
 #define cs_debug_trace(pos) do { } while (0)
@@ -196,7 +198,7 @@ static int cs_task_create(unsigned long clone_flags)
  * Returns 0 on success, negative value otherwise.
  */
 static int cs_cred_prepare(struct cred *new, const struct cred *old,
-			    gfp_t gfp)
+			   gfp_t gfp)
 {
 	int rc1;
 	/*
@@ -209,8 +211,8 @@ static int cs_cred_prepare(struct cred *new, const struct cred *old,
 	rc1 = cs_copy_cred_security(new, old, gfp);
 	if (gfp == GFP_KERNEL)
 		cs_task_security_gc();
-	if (original_security_ops.cred_prepare) {
-		const int rc2 = original_security_ops.cred_prepare(new, old,
+	if (original_cred_prepare.cred_prepare) {
+		const int rc2 = original_cred_prepare.cred_prepare(new, old,
 								   gfp);
 		if (rc2) {
 			cs_del_security(cs_find_cred_security(new));
@@ -229,8 +231,8 @@ static int cs_cred_prepare(struct cred *new, const struct cred *old,
  */
 static void cs_cred_free(struct cred *cred)
 {
-	if (original_security_ops.cred_free)
-		original_security_ops.cred_free(cred);
+	if (original_cred_free.cred_free)
+		original_cred_free.cred_free(cred);
 	cs_del_security(cs_find_cred_security(cred));
 }
 
@@ -264,9 +266,9 @@ static int cs_alloc_cred_security(const struct cred *cred, gfp_t gfp)
 static int cs_cred_alloc_blank(struct cred *new, gfp_t gfp)
 {
 	const int rc1 = cs_alloc_cred_security(new, gfp);
-	if (original_security_ops.cred_alloc_blank) {
-		const int rc2 = original_security_ops.cred_alloc_blank(new,
-								       gfp);
+	if (original_cred_alloc_blank.cred_alloc_blank) {
+		const int rc2 = original_cred_alloc_blank.
+			cred_alloc_blank(new, gfp);
 		if (rc2) {
 			cs_del_security(cs_find_cred_security(new));
 			return rc2;
@@ -412,7 +414,7 @@ static int cs_bprm_check_security(struct linux_binprm *bprm)
  */
 static int cs_file_open(struct file *f, const struct cred *cred)
 {
-	return cs_open_permission(f);
+	return cs_open_permission(&f->f_path, f->f_flags);
 }
 
 #ifdef CONFIG_SECURITY_PATH
@@ -428,7 +430,7 @@ static int cs_file_open(struct file *f, const struct cred *cred)
  */
 static int cs_path_chown(struct path *path, kuid_t user, kgid_t group)
 {
-	return cs_chown_permission(path->dentry, path->mnt, user, group);
+	return cs_chown_permission(path, user, group);
 }
 
 /**
@@ -441,7 +443,7 @@ static int cs_path_chown(struct path *path, kuid_t user, kgid_t group)
  */
 static int cs_path_chmod(struct path *path, umode_t mode)
 {
-	return cs_chmod_permission(path->dentry, path->mnt, mode);
+	return cs_chmod_permission(path, mode);
 }
 
 /**
@@ -465,7 +467,7 @@ static int cs_path_chroot(struct path *path)
  */
 static int cs_path_truncate(struct path *path)
 {
-	return cs_truncate_permission(path->dentry, path->mnt);
+	return cs_truncate_permission(path);
 }
 
 #else
@@ -480,23 +482,17 @@ static int cs_path_truncate(struct path *path)
  */
 static int cs_inode_setattr(struct dentry *dentry, struct iattr *attr)
 {
-	const int rc1 = (attr->ia_valid & ATTR_UID) ?
-		cs_chown_permission(dentry, NULL, attr->ia_uid, INVALID_GID) :
-		0;
-	const int rc2 = (attr->ia_valid & ATTR_GID) ?
-		cs_chown_permission(dentry, NULL, INVALID_UID, attr->ia_gid) :
-		0;
-	const int rc3 = (attr->ia_valid & ATTR_MODE) ?
-		cs_chmod_permission(dentry, NULL, attr->ia_mode) : 0;
-	const int rc4 = (attr->ia_valid & ATTR_SIZE) ?
-		cs_truncate_permission(dentry, NULL) : 0;
-	if (rc4)
-		return rc4;
-	if (rc3)
-		return rc3;
-	if (rc2)
-		return rc2;
-	return rc1;
+	int rc = 0;
+	struct path path = { NULL, dentry };
+	if (attr->ia_valid & ATTR_UID)
+		rc = cs_chown_permission(&path, attr->ia_uid, INVALID_GID);
+	if (!rc && (attr->ia_valid & ATTR_GID))
+		rc = cs_chown_permission(&path, INVALID_UID, attr->ia_gid);
+	if (!rc && (attr->ia_valid & ATTR_MODE))
+		rc = cs_chmod_permission(&path, attr->ia_mode);
+	if (!rc && (attr->ia_valid & ATTR_SIZE))
+		rc = cs_truncate_permission(&path);
+	return rc;
 }
 
 #endif
@@ -504,14 +500,13 @@ static int cs_inode_setattr(struct dentry *dentry, struct iattr *attr)
 /**
  * cs_inode_getattr - Check permission for stat().
  *
- * @mnt:    Pointer to "struct vfsmount".
- * @dentry: Pointer to "struct dentry".
+ * @path: Pointer to "struct path".
  *
  * Returns 0 on success, negative value otherwise.
  */
-static int cs_inode_getattr(struct vfsmount *mnt, struct dentry *dentry)
+static int cs_inode_getattr(const struct path *path)
 {
-	return cs_getattr_permission(mnt, dentry);
+	return cs_getattr_permission(path);
 }
 
 #ifdef CONFIG_SECURITY_PATH
@@ -527,9 +522,10 @@ static int cs_inode_getattr(struct vfsmount *mnt, struct dentry *dentry)
  * Returns 0 on success, negative value otherwise.
  */
 static int cs_path_mknod(struct path *dir, struct dentry *dentry,
-			  umode_t mode, unsigned int dev)
+			 umode_t mode, unsigned int dev)
 {
-	return cs_mknod_permission(dentry, dir->mnt, mode, dev);
+	struct path path = { dir->mnt, dentry };
+	return cs_mknod_permission(&path, mode, dev);
 }
 
 /**
@@ -542,9 +538,10 @@ static int cs_path_mknod(struct path *dir, struct dentry *dentry,
  * Returns 0 on success, negative value otherwise.
  */
 static int cs_path_mkdir(struct path *dir, struct dentry *dentry,
-			  umode_t mode)
+			 umode_t mode)
 {
-	return cs_mkdir_permission(dentry, dir->mnt, mode);
+	struct path path = { dir->mnt, dentry };
+	return cs_mkdir_permission(&path, mode);
 }
 
 /**
@@ -557,7 +554,8 @@ static int cs_path_mkdir(struct path *dir, struct dentry *dentry,
  */
 static int cs_path_rmdir(struct path *dir, struct dentry *dentry)
 {
-	return cs_rmdir_permission(dentry, dir->mnt);
+	struct path path = { dir->mnt, dentry };
+	return cs_rmdir_permission(&path);
 }
 
 /**
@@ -570,7 +568,8 @@ static int cs_path_rmdir(struct path *dir, struct dentry *dentry)
  */
 static int cs_path_unlink(struct path *dir, struct dentry *dentry)
 {
-	return cs_unlink_permission(dentry, dir->mnt);
+	struct path path = { dir->mnt, dentry };
+	return cs_unlink_permission(&path);
 }
 
 /**
@@ -583,9 +582,10 @@ static int cs_path_unlink(struct path *dir, struct dentry *dentry)
  * Returns 0 on success, negative value otherwise.
  */
 static int cs_path_symlink(struct path *dir, struct dentry *dentry,
-			    const char *old_name)
+			   const char *old_name)
 {
-	return cs_symlink_permission(dentry, dir->mnt, old_name);
+	struct path path = { dir->mnt, dentry };
+	return cs_symlink_permission(&path, old_name);
 }
 
 /**
@@ -601,7 +601,9 @@ static int cs_path_symlink(struct path *dir, struct dentry *dentry,
 static int cs_path_rename(struct path *old_dir, struct dentry *old_dentry,
 			  struct path *new_dir, struct dentry *new_dentry)
 {
-	return cs_rename_permission(old_dentry, new_dentry, old_dir->mnt);
+	struct path old = { old_dir->mnt, old_dentry };
+	struct path new = { new_dir->mnt, new_dentry };
+	return cs_rename_permission(&old, &new);
 }
 
 /**
@@ -614,9 +616,11 @@ static int cs_path_rename(struct path *old_dir, struct dentry *old_dentry,
  * Returns 0 on success, negative value otherwise.
  */
 static int cs_path_link(struct dentry *old_dentry, struct path *new_dir,
-			 struct dentry *new_dentry)
+			struct dentry *new_dentry)
 {
-	return cs_link_permission(old_dentry, new_dentry, new_dir->mnt);
+	struct path old = { new_dir->mnt, old_dentry };
+	struct path new = { new_dir->mnt, new_dentry };
+	return cs_link_permission(&old, &new);
 }
 
 #else
@@ -632,9 +636,10 @@ static int cs_path_link(struct dentry *old_dentry, struct path *new_dir,
  * Returns 0 on success, negative value otherwise.
  */
 static int cs_inode_mknod(struct inode *dir, struct dentry *dentry,
-			   umode_t mode, dev_t dev)
+			  umode_t mode, dev_t dev)
 {
-	return cs_mknod_permission(dentry, NULL, mode, dev);
+	struct path path = { NULL, dentry };
+	return cs_mknod_permission(&path, mode, dev);
 }
 
 /**
@@ -647,9 +652,10 @@ static int cs_inode_mknod(struct inode *dir, struct dentry *dentry,
  * Returns 0 on success, negative value otherwise.
  */
 static int cs_inode_mkdir(struct inode *dir, struct dentry *dentry,
-			   umode_t mode)
+			  umode_t mode)
 {
-	return cs_mkdir_permission(dentry, NULL, mode);
+	struct path path = { NULL, dentry };
+	return cs_mkdir_permission(&path, mode);
 }
 
 /**
@@ -662,7 +668,8 @@ static int cs_inode_mkdir(struct inode *dir, struct dentry *dentry,
  */
 static int cs_inode_rmdir(struct inode *dir, struct dentry *dentry)
 {
-	return cs_rmdir_permission(dentry, NULL);
+	struct path path = { NULL, dentry };
+	return cs_rmdir_permission(&path);
 }
 
 /**
@@ -675,7 +682,8 @@ static int cs_inode_rmdir(struct inode *dir, struct dentry *dentry)
  */
 static int cs_inode_unlink(struct inode *dir, struct dentry *dentry)
 {
-	return cs_unlink_permission(dentry, NULL);
+	struct path path = { NULL, dentry };
+	return cs_unlink_permission(&path);
 }
 
 /**
@@ -688,9 +696,10 @@ static int cs_inode_unlink(struct inode *dir, struct dentry *dentry)
  * Returns 0 on success, negative value otherwise.
  */
 static int cs_inode_symlink(struct inode *dir, struct dentry *dentry,
-			     const char *old_name)
+			    const char *old_name)
 {
-	return cs_symlink_permission(dentry, NULL, old_name);
+	struct path path = { NULL, dentry };
+	return cs_symlink_permission(&path, old_name);
 }
 
 /**
@@ -706,7 +715,9 @@ static int cs_inode_symlink(struct inode *dir, struct dentry *dentry,
 static int cs_inode_rename(struct inode *old_dir, struct dentry *old_dentry,
 			   struct inode *new_dir, struct dentry *new_dentry)
 {
-	return cs_rename_permission(old_dentry, new_dentry, NULL);
+	struct path old = { NULL, old_dentry };
+	struct path new = { NULL, new_dentry };
+	return cs_rename_permission(&old, &new);
 }
 
 /**
@@ -719,9 +730,11 @@ static int cs_inode_rename(struct inode *old_dir, struct dentry *old_dentry,
  * Returns 0 on success, negative value otherwise.
  */
 static int cs_inode_link(struct dentry *old_dentry, struct inode *dir,
-			  struct dentry *new_dentry)
+			 struct dentry *new_dentry)
 {
-	return cs_link_permission(old_dentry, new_dentry, NULL);
+	struct path old = { NULL, old_dentry };
+	struct path new = { NULL, new_dentry };
+	return cs_link_permission(&old, &new);
 }
 
 /**
@@ -734,9 +747,10 @@ static int cs_inode_link(struct dentry *old_dentry, struct inode *dir,
  * Returns 0 on success, negative value otherwise.
  */
 static int cs_inode_create(struct inode *dir, struct dentry *dentry,
-			    umode_t mode)
+			   umode_t mode)
 {
-	return cs_mknod_permission(dentry, NULL, mode, 0);
+	struct path path = { NULL, dentry };
+	return cs_mknod_permission(&path, mode, 0);
 }
 
 #endif
@@ -925,7 +939,7 @@ static int cs_socket_listen(struct socket *sock, int backlog)
  * Returns 0 on success, negative value otherwise.
  */
 static int cs_socket_connect(struct socket *sock, struct sockaddr *addr,
-			      int addr_len)
+			     int addr_len)
 {
 	const int rc = cs_validate_socket(sock);
 	if (rc < 0)
@@ -943,7 +957,7 @@ static int cs_socket_connect(struct socket *sock, struct sockaddr *addr,
  * Returns 0 on success, negative value otherwise.
  */
 static int cs_socket_bind(struct socket *sock, struct sockaddr *addr,
-			   int addr_len)
+			  int addr_len)
 {
 	const int rc = cs_validate_socket(sock);
 	if (rc < 0)
@@ -961,7 +975,7 @@ static int cs_socket_bind(struct socket *sock, struct sockaddr *addr,
  * Returns 0 on success, negative value otherwise.
  */
 static int cs_socket_sendmsg(struct socket *sock, struct msghdr *msg,
-			      int size)
+			     int size)
 {
 	const int rc = cs_validate_socket(sock);
 	if (rc < 0)
@@ -980,7 +994,7 @@ static int cs_socket_sendmsg(struct socket *sock, struct msghdr *msg,
  * Returns 0 on success, negative value otherwise.
  */
 static int cs_socket_recvmsg(struct socket *sock, struct msghdr *msg,
-			      int size, int flags)
+			     int size, int flags)
 {
 	return cs_validate_socket(sock);
 }
@@ -1094,7 +1108,7 @@ static int cs_sb_pivotroot(struct path *old_path, struct path *new_path)
  * Returns 0 on success, negative value otherwise.
  */
 static int cs_sb_mount(const char *dev_name, struct path *path,
-			const char *type, unsigned long flags, void *data_page)
+		       const char *type, unsigned long flags, void *data_page)
 {
 	return cs_mount_permission(dev_name, path, type, flags, data_page);
 }
@@ -1109,7 +1123,8 @@ static int cs_sb_mount(const char *dev_name, struct path *path,
  */
 static int cs_sb_umount(struct vfsmount *mnt, int flags)
 {
-	return cs_umount_permission(mnt, flags);
+	struct path path = { mnt, mnt->mnt_root };
+	return cs_umount_permission(&path, flags);
 }
 
 /**
@@ -1122,7 +1137,7 @@ static int cs_sb_umount(struct vfsmount *mnt, int flags)
  * Returns 0 on success, negative value otherwise.
  */
 static int cs_file_fcntl(struct file *file, unsigned int cmd,
-			  unsigned long arg)
+			 unsigned long arg)
 {
 	return cs_fcntl_permission(file, cmd, arg);
 }
@@ -1142,149 +1157,86 @@ static int cs_file_ioctl(struct file *filp, unsigned int cmd,
 	return cs_ioctl_permission(filp, cmd, arg);
 }
 
-static struct security_operations caitsith_security_ops = {
-	.name = "caitsith",
-	.task_create = cs_task_create,
-	.cred_prepare = cs_cred_prepare,
-	.cred_free = cs_cred_free,
-	.cred_alloc_blank = cs_cred_alloc_blank,
-	.cred_transfer = cs_cred_transfer,
-	.bprm_check_security = cs_bprm_check_security,
-	.bprm_committing_creds = cs_bprm_committing_creds,
-	.file_open = cs_file_open,
-	.file_fcntl = cs_file_fcntl,
-	.file_ioctl = cs_file_ioctl,
-	.sb_pivotroot = cs_sb_pivotroot,
-	.sb_mount = cs_sb_mount,
-	.sb_umount = cs_sb_umount,
+#define MY_HOOK_INIT(HEAD, HOOK)				\
+	{ .head = &probe_dummy_security_hook_heads.HEAD,	\
+			.hook = { .HEAD = HOOK } }
+
+static struct security_hook_list caitsith_hooks[] = {
+	/* Security context allocator. */
+	MY_HOOK_INIT(cred_free, cs_cred_free),
+	MY_HOOK_INIT(cred_prepare, cs_cred_prepare),
+	MY_HOOK_INIT(cred_alloc_blank, cs_cred_alloc_blank),
+	MY_HOOK_INIT(cred_transfer, cs_cred_transfer),
+	MY_HOOK_INIT(task_create, cs_task_create),
+	/* Security context updater for successful execve(). */
+	MY_HOOK_INIT(bprm_check_security, cs_bprm_check_security),
+	MY_HOOK_INIT(bprm_committing_creds, cs_bprm_committing_creds),
+	/* Various permission checker. */
+	MY_HOOK_INIT(file_open, cs_file_open),
+	MY_HOOK_INIT(file_fcntl, cs_file_fcntl),
+	MY_HOOK_INIT(file_ioctl, cs_file_ioctl),
+	MY_HOOK_INIT(sb_pivotroot, cs_sb_pivotroot),
+	MY_HOOK_INIT(sb_mount, cs_sb_mount),
+	MY_HOOK_INIT(sb_umount, cs_sb_umount),
 #ifdef CONFIG_SECURITY_PATH
-	.path_mknod = cs_path_mknod,
-	.path_mkdir = cs_path_mkdir,
-	.path_rmdir = cs_path_rmdir,
-	.path_unlink = cs_path_unlink,
-	.path_symlink = cs_path_symlink,
-	.path_rename = cs_path_rename,
-	.path_link = cs_path_link,
-	.path_truncate = cs_path_truncate,
-	.path_chmod = cs_path_chmod,
-	.path_chown = cs_path_chown,
-	.path_chroot = cs_path_chroot,
+	MY_HOOK_INIT(path_mknod, cs_path_mknod),
+	MY_HOOK_INIT(path_mkdir, cs_path_mkdir),
+	MY_HOOK_INIT(path_rmdir, cs_path_rmdir),
+	MY_HOOK_INIT(path_unlink, cs_path_unlink),
+	MY_HOOK_INIT(path_symlink, cs_path_symlink),
+	MY_HOOK_INIT(path_rename, cs_path_rename),
+	MY_HOOK_INIT(path_link, cs_path_link),
+	MY_HOOK_INIT(path_truncate, cs_path_truncate),
+	MY_HOOK_INIT(path_chmod, cs_path_chmod),
+	MY_HOOK_INIT(path_chown, cs_path_chown),
+	MY_HOOK_INIT(path_chroot, cs_path_chroot),
 #else
-	.inode_mknod = cs_inode_mknod,
-	.inode_mkdir = cs_inode_mkdir,
-	.inode_rmdir = cs_inode_rmdir,
-	.inode_unlink = cs_inode_unlink,
-	.inode_symlink = cs_inode_symlink,
-	.inode_rename = cs_inode_rename,
-	.inode_link = cs_inode_link,
-	.inode_create = cs_inode_create,
-	.inode_setattr = cs_inode_setattr,
+	MY_HOOK_INIT(inode_mknod, cs_inode_mknod),
+	MY_HOOK_INIT(inode_mkdir, cs_inode_mkdir),
+	MY_HOOK_INIT(inode_rmdir, cs_inode_rmdir),
+	MY_HOOK_INIT(inode_unlink, cs_inode_unlink),
+	MY_HOOK_INIT(inode_symlink, cs_inode_symlink),
+	MY_HOOK_INIT(inode_rename, cs_inode_rename),
+	MY_HOOK_INIT(inode_link, cs_inode_link),
+	MY_HOOK_INIT(inode_create, cs_inode_create),
+	MY_HOOK_INIT(inode_setattr, cs_inode_setattr),
 #endif
-	.inode_getattr = cs_inode_getattr,
+	MY_HOOK_INIT(inode_getattr, cs_inode_getattr),
 #ifdef CONFIG_SECURITY_NETWORK
-	.socket_bind = cs_socket_bind,
-	.socket_connect = cs_socket_connect,
-	.socket_listen = cs_socket_listen,
-	.socket_sendmsg = cs_socket_sendmsg,
-	.socket_recvmsg = cs_socket_recvmsg,
-	.socket_getsockname = cs_socket_getsockname,
-	.socket_getpeername = cs_socket_getpeername,
-	.socket_getsockopt = cs_socket_getsockopt,
-	.socket_setsockopt = cs_socket_setsockopt,
-	.socket_shutdown = cs_socket_shutdown,
-	.socket_accept = cs_socket_accept,
-	.inode_free_security = cs_inode_free_security,
+	MY_HOOK_INIT(socket_bind, cs_socket_bind),
+	MY_HOOK_INIT(socket_connect, cs_socket_connect),
+	MY_HOOK_INIT(socket_listen, cs_socket_listen),
+	MY_HOOK_INIT(socket_sendmsg, cs_socket_sendmsg),
+	MY_HOOK_INIT(socket_recvmsg, cs_socket_recvmsg),
+	MY_HOOK_INIT(socket_getsockname, cs_socket_getsockname),
+	MY_HOOK_INIT(socket_getpeername, cs_socket_getpeername),
+	MY_HOOK_INIT(socket_getsockopt, cs_socket_getsockopt),
+	MY_HOOK_INIT(socket_setsockopt, cs_socket_setsockopt),
+	MY_HOOK_INIT(socket_shutdown, cs_socket_shutdown),
+	MY_HOOK_INIT(socket_accept, cs_socket_accept),
+	MY_HOOK_INIT(inode_free_security, cs_inode_free_security),
 #endif
 };
 
-#define swap_security_ops(op, lsm_list)					\
-	do {								\
-		if (list_empty(&lsm_list[lsm_##op]))			\
-			add_security_ops(op, lsm_list);			\
-		else {							\
-			struct security_operations *ops =		\
-				list_first_entry(&lsm_list[lsm_##op],	\
-						 typeof(*ops),		\
-						 list[lsm_##op]);	\
-			original_security_ops.op = ops->op;		\
-			smp_wmb();					\
-			ops->op = cs_##op;				\
-		}							\
-	} while (0)
-
-#define add_security_ops(op, lsm_list)					\
-	do {								\
-		list_add_tail_rcu(&caitsith_security_ops.list[lsm_##op],	\
-				  &lsm_list[lsm_##op]);			\
-	} while (0)
-
-/**
- * cs_update_security_ops - Overwrite original "struct security_operations".
- *
- * @lsm_list: Pointer to "struct list_head lsm_hooks[LSM_MAX_HOOKS]".
- *
- * Returns nothing.
- */
-static void __init cs_update_security_ops(struct list_head *lsm_list)
+static inline void add_hook(struct security_hook_list *hook)
 {
-	/* Security context allocator. */
-	swap_security_ops(cred_free, lsm_list);
-	swap_security_ops(cred_prepare, lsm_list);
-	swap_security_ops(cred_alloc_blank, lsm_list);
-	add_security_ops(cred_transfer, lsm_list);
-	add_security_ops(task_create, lsm_list);
-	/* Security context updater for successful execve(). */
-	add_security_ops(bprm_check_security, lsm_list);
-	add_security_ops(bprm_committing_creds, lsm_list);
-	/* Various permission checker. */
-	add_security_ops(file_open, lsm_list);
-	add_security_ops(file_fcntl, lsm_list);
-	add_security_ops(file_ioctl, lsm_list);
-	add_security_ops(sb_pivotroot, lsm_list);
-	add_security_ops(sb_mount, lsm_list);
-	add_security_ops(sb_umount, lsm_list);
-#ifdef CONFIG_SECURITY_PATH
-	add_security_ops(path_mknod, lsm_list);
-	add_security_ops(path_mkdir, lsm_list);
-	add_security_ops(path_rmdir, lsm_list);
-	add_security_ops(path_unlink, lsm_list);
-	add_security_ops(path_symlink, lsm_list);
-	add_security_ops(path_rename, lsm_list);
-	add_security_ops(path_link, lsm_list);
-	add_security_ops(path_truncate, lsm_list);
-	add_security_ops(path_chmod, lsm_list);
-	add_security_ops(path_chown, lsm_list);
-	add_security_ops(path_chroot, lsm_list);
-#else
-	add_security_ops(inode_mknod, lsm_list);
-	add_security_ops(inode_mkdir, lsm_list);
-	add_security_ops(inode_rmdir, lsm_list);
-	add_security_ops(inode_unlink, lsm_list);
-	add_security_ops(inode_symlink, lsm_list);
-	add_security_ops(inode_rename, lsm_list);
-	add_security_ops(inode_link, lsm_list);
-	add_security_ops(inode_create, lsm_list);
-	add_security_ops(inode_setattr, lsm_list);
-#endif
-	add_security_ops(inode_getattr, lsm_list);
-#ifdef CONFIG_SECURITY_NETWORK
-	add_security_ops(inode_free_security, lsm_list);
-	add_security_ops(socket_bind, lsm_list);
-	add_security_ops(socket_connect, lsm_list);
-	add_security_ops(socket_listen, lsm_list);
-	add_security_ops(socket_sendmsg, lsm_list);
-	add_security_ops(socket_recvmsg, lsm_list);
-	add_security_ops(socket_getsockname, lsm_list);
-	add_security_ops(socket_getpeername, lsm_list);
-	add_security_ops(socket_getsockopt, lsm_list);
-	add_security_ops(socket_setsockopt, lsm_list);
-	add_security_ops(socket_shutdown, lsm_list);
-	add_security_ops(socket_accept, lsm_list);
-#endif
+	list_add_tail_rcu(&hook->list, hook->head);
 }
 
-#undef swap_security_ops
-#undef add_security_ops
+static void __init swap_hook(struct security_hook_list *hook,
+			     union security_list_options *original)
+{
+	struct list_head *list = hook->head;
+	if (list_empty(list)) {
+		add_hook(hook);
+	} else {
+		struct security_hook_list *shp =
+			list_last_entry(list, struct security_hook_list, list);
+		*original = shp->hook;
+		smp_wmb();
+		shp->hook = hook->hook;
+	}
+}
 
 /**
  * cs_init - Initialize this module.
@@ -1294,9 +1246,13 @@ static void __init cs_update_security_ops(struct list_head *lsm_list)
 static int __init cs_init(void)
 {
 	int idx;
-	struct list_head *hooks = probe_lsm_hooks_list();
+	struct security_hook_heads *hooks = probe_security_hook_heads();
 	if (!hooks)
 		goto out;
+	for (idx = 0; idx < ARRAY_SIZE(caitsith_hooks); idx++)
+		caitsith_hooks[idx].head = ((void *) hooks)
+			+ ((unsigned long) caitsith_hooks[idx].head)
+			- ((unsigned long) &probe_dummy_security_hook_heads);
 	caitsith_exports.find_task_by_vpid = probe_find_task_by_vpid();
 	if (!caitsith_exports.find_task_by_vpid)
 		goto out;
@@ -1311,7 +1267,11 @@ static int __init cs_init(void)
 		INIT_LIST_HEAD(&cs_task_security_list[idx]);
 	}
 	cs_init_module();
-	cs_update_security_ops(hooks);
+	swap_hook(&caitsith_hooks[0], &original_cred_free);
+	swap_hook(&caitsith_hooks[1], &original_cred_prepare);
+	swap_hook(&caitsith_hooks[2], &original_cred_alloc_blank);
+	for (idx = 3; idx < ARRAY_SIZE(caitsith_hooks); idx++)
+		add_hook(&caitsith_hooks[idx]);
 	return 0;
 out:
 	return -EINVAL;
@@ -1355,7 +1315,7 @@ bool cs_used_by_cred(const struct cs_domain_info *domain)
  * Returns nothing.
  */
 static void cs_add_task_security(struct cs_security *ptr,
-				  struct list_head *list)
+				 struct list_head *list)
 {
 	unsigned long flags;
 	spin_lock_irqsave(&cs_task_security_list_lock, flags);
