@@ -3,7 +3,7 @@
  *
  * Copyright (C) 2005-2012  NTT DATA CORPORATION
  *
- * Version: 0.1.11   2015/01/12
+ * Version: 0.1.12   2015/04/08
  */
 
 #include "caitsith.h"
@@ -127,16 +127,17 @@ static bool cs_path_matches_group(const struct cs_path_info *pathname,
 static int cs_execute_path(struct linux_binprm *bprm, struct path *path);
 static int cs_execute(struct cs_request_info *r);
 static int cs_kern_path(const char *pathname, int flags, struct path *path);
-static int cs_mkdev_perm(const u8 operation, struct path *path,
+static int cs_mkdev_perm(const u8 operation, const struct path *path,
 			 const unsigned int mode, unsigned int dev);
-static int cs_mount_acl(const char *dev_name, struct path *dir,
+static int cs_mount_acl(const char *dev_name, const struct path *dir,
 			const char *type, unsigned long flags,
 			const char *data);
-static int cs_path2_perm(const enum cs_mac_index operation, struct path *path1,
-			 struct path *path2);
-static int cs_path_number_perm(const enum cs_mac_index type, struct path *path,
-			       unsigned long number);
-static int cs_path_perm(const enum cs_mac_index operation, struct path *path);
+static int cs_path2_perm(const enum cs_mac_index operation,
+			 const struct path *path1, const struct path *path2);
+static int cs_path_number_perm(const enum cs_mac_index type,
+			       const struct path *path, unsigned long number);
+static int cs_path_perm(const enum cs_mac_index operation,
+			const struct path *path);
 int cs_start_execve(struct linux_binprm *bprm, struct cs_request_info **rp);
 static void cs_check_auto_domain_transition(void);
 static void cs_clear_request_info(struct cs_request_info *r);
@@ -951,7 +952,7 @@ static int cs_execute_path(struct linux_binprm *bprm, struct path *path)
  *
  * Returns 0 on success, negative value otherwise.
  */
-static int cs_mount_acl(const char *dev_name, struct path *dir,
+static int cs_mount_acl(const char *dev_name, const struct path *dir,
 			const char *type, unsigned long flags,
 			const char *data)
 {
@@ -1050,7 +1051,7 @@ out:
  *
  * Returns 0 on success, negative value otherwise.
  */
-int cs_mount_permission(const char *dev_name, struct path *path,
+int cs_mount_permission(const char *dev_name, const struct path *path,
 			const char *type, unsigned long flags,
 			void *data_page)
 {
@@ -1103,7 +1104,7 @@ int cs_mount_permission(const char *dev_name, struct path *path,
  *
  * Returns 0 on success, negative value otherwise.
  */
-int cs_open_permission(struct path *path, const int flag)
+int cs_open_permission(const struct path *path, const int flag)
 {
 	struct cs_request_info r = { };
 	const u32 cs_flags = cs_current_flags();
@@ -1118,9 +1119,14 @@ int cs_open_permission(struct path *path, const int flag)
 	if (current->in_execve && !(cs_flags & CS_TASK_IS_IN_EXECVE))
 		return 0;
 #endif
-#ifndef CONFIG_CAITSITH_GETATTR
+#ifndef CONFIG_CAITSITH_READDIR
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(4, 1, 0)
+	if (d_is_dir(path->dentry))
+		return 0;
+#else
 	if (path->dentry->d_inode && S_ISDIR(path->dentry->d_inode->i_mode))
 		return 0;
+#endif
 #endif
 	r.obj.path[0] = *path;
 	if (!(cs_flags & CS_TASK_IS_IN_EXECVE))
@@ -1151,7 +1157,8 @@ int cs_open_permission(struct path *path, const int flag)
  *
  * Returns 0 on success, negative value otherwise.
  */
-static int cs_path_perm(const enum cs_mac_index operation, struct path *path)
+static int cs_path_perm(const enum cs_mac_index operation,
+			const struct path *path)
 {
 	struct cs_request_info r = { };
 	cs_check_auto_domain_transition();
@@ -1170,7 +1177,7 @@ static int cs_path_perm(const enum cs_mac_index operation, struct path *path)
  *
  * Returns 0 on success, negative value otherwise.
  */
-static int cs_mkdev_perm(const u8 operation, struct path *path,
+static int cs_mkdev_perm(const u8 operation, const struct path *path,
 			 const unsigned int mode, unsigned int dev)
 {
 	struct cs_request_info r = { };
@@ -1195,8 +1202,8 @@ static int cs_mkdev_perm(const u8 operation, struct path *path,
  *
  * Returns 0 on success, negative value otherwise.
  */
-static int cs_path2_perm(const enum cs_mac_index operation, struct path *path1,
-			 struct path *path2)
+static int cs_path2_perm(const enum cs_mac_index operation,
+			 const struct path *path1, const struct path *path2)
 {
 	struct cs_request_info r = { };
 	cs_check_auto_domain_transition();
@@ -1214,7 +1221,7 @@ static int cs_path2_perm(const enum cs_mac_index operation, struct path *path1,
  *
  * Returns 0 on success, negative value otherwise.
  */
-int cs_symlink_permission(struct path *path, const char *target)
+int cs_symlink_permission(const struct path *path, const char *target)
 {
 	struct cs_request_info r = { };
 	cs_check_auto_domain_transition();
@@ -1237,8 +1244,8 @@ int cs_symlink_permission(struct path *path, const char *target)
  *
  * Returns 0 on success, negative value otherwise.
  */
-static int cs_path_number_perm(const enum cs_mac_index type, struct path *path,
-			       unsigned long number)
+static int cs_path_number_perm(const enum cs_mac_index type,
+			       const struct path *path, unsigned long number)
 {
 	struct cs_request_info r = { };
 	cs_check_auto_domain_transition();
@@ -1271,7 +1278,7 @@ int cs_ioctl_permission(struct file *filp, unsigned int cmd,
  *
  * Returns 0 on success, negative value otherwise.
  */
-int cs_chmod_permission(struct path *path, mode_t mode)
+int cs_chmod_permission(const struct path *path, mode_t mode)
 {
 	return cs_path_number_perm(CS_MAC_CHMOD, path, mode & S_IALLUGO);
 }
@@ -1287,7 +1294,7 @@ int cs_chmod_permission(struct path *path, mode_t mode)
  *
  * Returns 0 on success, negative value otherwise.
  */
-int cs_chown_permission(struct path *path, kuid_t user, kgid_t group)
+int cs_chown_permission(const struct path *path, kuid_t user, kgid_t group)
 {
 	int error = 0;
 	if (uid_valid(user))
@@ -1310,7 +1317,7 @@ int cs_chown_permission(struct path *path, kuid_t user, kgid_t group)
  *
  * Returns 0 on success, negative value otherwise.
  */
-int cs_chown_permission(struct path *path, uid_t user, gid_t group)
+int cs_chown_permission(const struct path *path, uid_t user, gid_t group)
 {
 	int error = 0;
 	if (user == (uid_t) -1 && group == (gid_t) -1)
@@ -1356,7 +1363,8 @@ int cs_fcntl_permission(struct file *file, unsigned int cmd,
  *
  * Returns 0 on success, negative value otherwise.
  */
-int cs_pivot_root_permission(struct path *old_path, struct path *new_path)
+int cs_pivot_root_permission(const struct path *old_path,
+			     const struct path *new_path)
 {
 	return cs_path2_perm(CS_MAC_PIVOT_ROOT, new_path, old_path);
 }
@@ -1368,7 +1376,7 @@ int cs_pivot_root_permission(struct path *old_path, struct path *new_path)
  *
  * Returns 0 on success, negative value otherwise.
  */
-int cs_chroot_permission(struct path *path)
+int cs_chroot_permission(const struct path *path)
 {
 	return cs_path_perm(CS_MAC_CHROOT, path);
 }
@@ -1381,7 +1389,7 @@ int cs_chroot_permission(struct path *path)
  *
  * Returns 0 on success, negative value otherwise.
  */
-int cs_umount_permission(struct path *path, int flags)
+int cs_umount_permission(const struct path *path, int flags)
 {
 	return cs_path_number_perm(CS_MAC_UMOUNT, path, flags);
 }
@@ -1395,7 +1403,7 @@ int cs_umount_permission(struct path *path, int flags)
  *
  * Returns 0 on success, negative value otherwise.
  */
-int cs_mknod_permission(struct path *path, const unsigned int mode,
+int cs_mknod_permission(const struct path *path, const unsigned int mode,
 			unsigned int dev)
 {
 	int error = 0;
@@ -1429,7 +1437,7 @@ int cs_mknod_permission(struct path *path, const unsigned int mode,
  *
  * Returns 0 on success, negative value otherwise.
  */
-int cs_mkdir_permission(struct path *path, unsigned int mode)
+int cs_mkdir_permission(const struct path *path, unsigned int mode)
 {
 	return cs_path_number_perm(CS_MAC_MKDIR, path, mode);
 }
@@ -1441,7 +1449,7 @@ int cs_mkdir_permission(struct path *path, unsigned int mode)
  *
  * Returns 0 on success, negative value otherwise.
  */
-int cs_rmdir_permission(struct path *path)
+int cs_rmdir_permission(const struct path *path)
 {
 	return cs_path_perm(CS_MAC_RMDIR, path);
 }
@@ -1453,7 +1461,7 @@ int cs_rmdir_permission(struct path *path)
  *
  * Returns 0 on success, negative value otherwise.
  */
-int cs_unlink_permission(struct path *path)
+int cs_unlink_permission(const struct path *path)
 {
 	return cs_path_perm(CS_MAC_UNLINK, path);
 }
@@ -1467,7 +1475,7 @@ int cs_unlink_permission(struct path *path)
  *
  * Returns 0 on success, negative value otherwise.
  */
-int cs_getattr_permission(struct path *path)
+int cs_getattr_permission(const struct path *path)
 {
 	return cs_path_perm(CS_MAC_GETATTR, path);
 }
@@ -1481,7 +1489,7 @@ int cs_getattr_permission(struct path *path)
  *
  * Returns 0 on success, negative value otherwise.
  */
-int cs_truncate_permission(struct path *path)
+int cs_truncate_permission(const struct path *path)
 {
 	return cs_path_perm(CS_MAC_TRUNCATE, path);
 }
@@ -1494,7 +1502,7 @@ int cs_truncate_permission(struct path *path)
  *
  * Returns 0 on success, negative value otherwise.
  */
-int cs_rename_permission(struct path *old, struct path *new)
+int cs_rename_permission(const struct path *old, const struct path *new)
 {
 	return cs_path2_perm(CS_MAC_RENAME, old, new);
 }
@@ -1507,7 +1515,7 @@ int cs_rename_permission(struct path *old, struct path *new)
  *
  * Returns 0 on success, negative value otherwise.
  */
-int cs_link_permission(struct path *old, struct path *new)
+int cs_link_permission(const struct path *old, const struct path *new)
 {
 	return cs_path2_perm(CS_MAC_LINK, old, new);
 }
