@@ -3,7 +3,7 @@
  *
  * Copyright (C) 2005-2012  NTT DATA CORPORATION
  *
- * Version: 0.1.11   2015/01/12
+ * Version: 0.1.12   2015/04/08
  */
 
 #include "caitsith.h"
@@ -20,13 +20,13 @@
 
 /***** SECTION3: Prototype definition section *****/
 
-static char *cs_get_absolute_path(struct path *path, char * const buffer,
+static char *cs_get_absolute_path(const struct path *path, char * const buffer,
 				  const int buflen);
 static char *cs_get_dentry_path(struct dentry *dentry, char * const buffer,
 				const int buflen);
 static char *cs_get_local_path(struct dentry *dentry, char * const buffer,
 			       const int buflen);
-static char *cs_get_socket_name(struct path *path, char * const buffer,
+static char *cs_get_socket_name(const struct path *path, char * const buffer,
 				const int buflen);
 static int cs_const_part_length(const char *filename);
 
@@ -160,7 +160,7 @@ static inline void cs_realpath_unlock(void)
  * Caller holds the dcache_lock and vfsmount_lock.
  * Based on __d_path() in fs/dcache.c
  */
-static char *cs_get_absolute_path(struct path *path, char * const buffer,
+static char *cs_get_absolute_path(const struct path *path, char * const buffer,
 				  const int buflen)
 {
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(3, 2, 0)
@@ -387,7 +387,7 @@ out:
  *
  * Returns the buffer.
  */
-static char *cs_get_socket_name(struct path *path, char * const buffer,
+static char *cs_get_socket_name(const struct path *path, char * const buffer,
 				const int buflen)
 {
 	struct inode *inode = path->dentry->d_inode;
@@ -413,7 +413,7 @@ static char *cs_get_socket_name(struct path *path, char * const buffer,
  * This function uses kzalloc(), so caller must kfree() if this function
  * didn't return NULL.
  */
-char *cs_realpath(struct path *path)
+char *cs_realpath(const struct path *path)
 {
 	char *buf = NULL;
 	char *name = NULL;
@@ -612,34 +612,36 @@ void cs_fill_path_info(struct cs_path_info *ptr)
 char *cs_get_exe(void)
 {
 	struct mm_struct *mm;
-#if LINUX_VERSION_CODE < KERNEL_VERSION(3, 7, 0)
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 26)
 	struct vm_area_struct *vma;
 #endif
-	bool done = false;
-	char *cp = NULL;
+	struct file *exe_file;
 	if (current->flags & PF_KTHREAD)
 		return kstrdup("<kernel>", GFP_NOFS);
 	mm = current->mm;
 	if (!mm)
 		goto task_has_no_mm;
 	down_read(&mm->mmap_sem);
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(3, 7, 0)
-	if (mm->exe_file) {
-		cp = cs_realpath(&mm->exe_file->f_path);
-		done = true;
-	}
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 26)
+	/* Not using get_mm_exe_file() as it is not exported. */
+	exe_file = mm->exe_file;
 #else
+	exe_file = NULL;
 	for (vma = mm->mmap; vma; vma = vma->vm_next) {
 		if ((vma->vm_flags & VM_EXECUTABLE) && vma->vm_file) {
-			cp = cs_realpath(&vma->vm_file->f_path);
-			done = true;
+			exe_file = vma->vm_file;
 			break;
 		}
 	}
 #endif
+	if (exe_file)
+		get_file(exe_file);
 	up_read(&mm->mmap_sem);
-	if (done)
+	if (exe_file) {
+		char *cp = cs_realpath(&exe_file->f_path);
+		fput(exe_file);
 		return cp;
+	}
 task_has_no_mm:
 	/* I'don't know. */
 	return kstrdup("<unknown>", GFP_NOFS);
